@@ -7,6 +7,7 @@ end-pp-include*)
 open Printf
 open Unix
 open Bigarray
+open Common
 
 type t = (char, int8_unsigned_elt, c_layout) Array1.t
 
@@ -20,10 +21,10 @@ let () =
   init ()
 
 let create n = Array1.create Bigarray.char c_layout n
-let length bstr = Array1.dim bstr
+let length (bstr : t) = Array1.dim bstr
 external is_mmapped : t -> bool = "bigstring_is_mmapped_stub" "noalloc"
 
-let check_args ~loc bstr ~pos ~len =
+let check_args ~loc ~pos ~len (bstr : t) =
   if pos < 0 then invalid_arg (loc ^ ": pos < 0");
   if len < 0 then invalid_arg (loc ^ ": len < 0");
   let bstr_len = length bstr in
@@ -31,10 +32,10 @@ let check_args ~loc bstr ~pos ~len =
     invalid_arg (sprintf "Bigstring.%s: length(bstr) < pos + len" loc)
 
 let get_opt_pos ~loc ~var = function
+  | None -> 0
   | Some pos ->
       if pos < 0 then invalid_arg (sprintf "Bigstring.%s: %s < 0" loc var);
       pos
-  | None -> 0
 
 let get_opt_len bstr ~pos = function
   | Some len -> len
@@ -51,96 +52,82 @@ let check_min_len ~loc ~len = function
         invalid_arg msg);
       min_len
 
-let sub ?(pos = 0) ?len bstr =
-  let len =
-    match len with
-    | None -> length bstr - pos
-    | Some len -> len
-  in
+let sub_shared ?(pos = 0) ?len (bstr : t) =
+  let len = get_opt_len bstr ~pos len in
   Array1.sub bstr pos len
 
 
 (* Blitting *)
 
 external unsafe_blit :
-  src_pos : int -> src : t -> dst_pos : int -> dst : t -> len : int -> unit
+  src : t -> src_pos : int -> dst : t -> dst_pos : int -> len : int -> unit
   = "bigstring_blit_stub"
 
-let blit ?src_pos ~src ?dst_pos ~dst len =
-  let loc = "blit" in
-  let src_pos = get_opt_pos ~loc ~var:"src_pos" src_pos in
-  let dst_pos = get_opt_pos ~loc ~var:"dst_pos" dst_pos in
-  if len < 0 then invalid_arg "Bigstring.blit: len < 0"
-  else if len = 0 then (
-    if src_pos > length src then
-      invalid_arg "Bigstring.blit: src_pos > src_len";
-    if dst_pos > length dst then
-      invalid_arg "Bigstring.blit: dst_pos > dst_len")
-  else (
-    if src_pos + len > length src then
-      invalid_arg "Bigstring.blit: src_pos + len > src_len"
-    else if dst_pos + len > length dst then
-      invalid_arg "Bigstring.blit: dst_pos + len > dst_len"
-    else unsafe_blit ~src_pos ~src ~dst_pos ~dst ~len)
+let blit_common
+    ~loc ~get_src_len ~get_dst_len ~blit ~src ~src_pos ~dst ~dst_pos ~len =
+  if len < 0 then invalid_argf "%s: len < 0" loc ()
+  else
+    let check_pos var total_len pos =
+      if pos < 0 then invalid_argf "%s: %s < 0" loc var ()
+      else if pos + len > total_len then
+        invalid_argf "%s: pos (%d) + len (%d) > total_len (%d)"
+          loc pos len total_len ()
+    in
+    check_pos "src_pos" (get_src_len src) src_pos;
+    check_pos "dst_pos" (get_dst_len dst) dst_pos;
+    if len > 0 then blit ~src ~src_pos ~dst ~dst_pos ~len
+
+let blit ~src ~src_pos ~dst ~dst_pos ~len =
+  blit_common
+    ~loc:"blit"
+    ~get_src_len:length ~get_dst_len:length
+    ~blit:unsafe_blit
+    ~src ~src_pos ~dst ~dst_pos ~len
+
+let sub_copy ?(pos = 0) ?len bstr =
+  let len = get_opt_len bstr ~pos len in
+  let dst = create len in
+  blit ~src:bstr ~src_pos:0 ~dst ~dst_pos:0 ~len;
+  dst
 
 external unsafe_blit_string_bigstring :
-  src_pos : int -> string -> dst_pos : int -> t -> len : int -> unit
+  src : string -> src_pos : int -> dst : t -> dst_pos : int -> len : int -> unit
   = "bigstring_blit_string_bigstring_stub" "noalloc"
 
-let blit_string_bigstring ?src_pos str ?dst_pos bstr ~len =
-  let loc = "blit_string_bigstring" in
-  let src_pos = get_opt_pos ~loc ~var:"src_pos" src_pos in
-  let dst_pos = get_opt_pos ~loc ~var:"dst_pos" dst_pos in
-  if len < 0 then invalid_arg "Bigstring.blit_string_bigstring: len < 0"
-  else if len = 0 then (
-    if src_pos > String.length str then
-      invalid_arg "Bigstring.blit_string_bigstring: src_pos > str_len";
-    if dst_pos > length bstr then
-      invalid_arg "Bigstring.blit_string_bigstring: src_pos > bstr_len")
-  else (
-    if src_pos + len > String.length str then
-      invalid_arg "Bigstring.blit_string_bigstring: src_pos + len > str_len"
-    else if dst_pos + len > length bstr then
-      invalid_arg "Bigstring.blit_string_bigstring: src_pos + len > bstr_len"
-    else unsafe_blit_string_bigstring ~src_pos str ~dst_pos bstr ~len)
+let blit_string_bigstring ~src ~src_pos ~dst ~dst_pos ~len =
+  blit_common
+    ~loc:"blit_string_bigstring"
+    ~get_src_len:String.length ~get_dst_len:length
+    ~blit:unsafe_blit_string_bigstring
+    ~src ~src_pos ~dst ~dst_pos ~len
 
-let of_string ?(pos = 0) ?len str =
+external unsafe_blit_bigstring_string :
+  src : t -> src_pos : int -> dst : string -> dst_pos : int -> len : int -> unit
+  = "bigstring_blit_bigstring_string_stub" "noalloc"
+
+let blit_bigstring_string ~src ~src_pos ~dst ~dst_pos ~len =
+  blit_common
+    ~loc:"blit_bigstring_string"
+    ~get_src_len:length ~get_dst_len:String.length
+    ~blit:unsafe_blit_bigstring_string
+    ~src ~src_pos ~dst ~dst_pos ~len
+
+let of_string ?(pos = 0) ?len src =
   let len =
     match len with
     | Some len -> len
-    | None -> String.length str - pos
+    | None -> String.length src - pos
   in
-  let bstr = create len in
-  blit_string_bigstring ~src_pos:pos str bstr ~len;
-  bstr
+  let dst = create len in
+  blit_string_bigstring ~src ~src_pos:pos ~dst ~dst_pos:0 ~len;
+  dst
 
-external unsafe_blit_bigstring_string :
-  src_pos : int -> t -> dst_pos : int -> string -> len : int -> unit
-  = "bigstring_blit_bigstring_string_stub" "noalloc"
-
-let blit_bigstring_string ?src_pos bstr ?dst_pos str ~len =
-  let loc = "blit_bigstring_string" in
-  let src_pos = get_opt_pos ~loc ~var:"src_pos" src_pos in
-  let dst_pos = get_opt_pos ~loc ~var:"dst_pos" dst_pos in
-  if len < 0 then invalid_arg "Bigstring.blit_bigstring_string: len < 0"
-  else if len = 0 then (
-    if src_pos > length bstr then
-      invalid_arg "Bigstring.blit_bigstring_string: src_pos > bstr_len";
-    if dst_pos > String.length str then
-      invalid_arg "Bigstring.blit_bigstring_string: src_pos > str_len")
-  else (
-    if src_pos + len > length bstr then
-      invalid_arg "Bigstring.blit_bigstring_string: src_pos + len > bstr_len"
-    else if dst_pos + len > String.length str then
-      invalid_arg "Bigstring.blit_bigstring_string: src_pos + len > str_len"
-    else unsafe_blit_bigstring_string ~src_pos bstr ~dst_pos str ~len)
-
-let to_string ?(pos = 0) ?len bstr =
-  let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"to_string" bstr ~pos ~len;
-  let str = String.create len in
-  blit_bigstring_string ~src_pos:pos bstr str ~len;
-  str
+let to_string ?(pos = 0) ?len src =
+  let len = get_opt_len src ~pos len in
+  check_args ~loc:"to_string" ~pos ~len src;
+  let dst = String.create len in
+  blit_bigstring_string ~src ~src_pos:pos ~dst ~dst_pos:0 ~len;
+  dst
 
 
 (* Input functions *)
@@ -152,7 +139,7 @@ external unsafe_read :
 let read ?min_len fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
   let loc = "read" in
-  check_args ~loc bstr ~pos ~len;
+  check_args ~loc ~pos ~len bstr;
   let min_len = check_min_len ~loc ~len min_len in
   unsafe_read ~min_len fd ~pos ~len bstr
 
@@ -166,17 +153,17 @@ external unsafe_really_recv :
 
 let really_recv sock ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"really_recv" bstr ~pos ~len;
+  check_args ~loc:"really_recv" ~pos ~len bstr;
   unsafe_really_recv sock ~pos ~len bstr
 
-external unsafe_read_assume_nonblocking :
+external unsafe_read_assume_fd_is_nonblocking :
   file_descr -> pos : int -> len : int -> t -> int
-  = "bigstring_read_assume_nonblocking_stub"
+  = "bigstring_read_assume_fd_is_nonblocking_stub"
 
-let read_assume_nonblocking fd ?(pos = 0) ?len bstr =
+let read_assume_fd_is_nonblocking fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"read_assume_nonblocking" bstr ~pos ~len;
-  unsafe_read_assume_nonblocking fd ~pos ~len bstr
+  check_args ~loc:"read_assume_fd_is_nonblocking" ~pos ~len bstr;
+  unsafe_read_assume_fd_is_nonblocking fd ~pos ~len bstr
 
 external unsafe_input :
   min_len : int -> in_channel -> pos : int -> len : int -> t -> int
@@ -185,15 +172,18 @@ external unsafe_input :
 let input ?min_len ic ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
   let loc = "input" in
-  check_args ~loc bstr ~pos ~len;
+  check_args ~loc ~pos ~len bstr;
   let min_len = check_min_len ~loc ~len min_len in
   unsafe_input ~min_len ic ~pos ~len bstr
 
 let really_input ic ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"really_input" bstr ~pos ~len;
+  check_args ~loc:"really_input" ~pos ~len bstr;
   ignore (unsafe_input ~min_len:len ic ~pos ~len bstr)
 
+
+(* Output functions *)
+
 external unsafe_output :
   min_len : int -> out_channel -> pos : int -> len : int -> t -> int
   = "bigstring_output_stub"
@@ -201,25 +191,22 @@ external unsafe_output :
 let output ?min_len oc ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
   let loc = "output" in
-  check_args ~loc bstr ~pos ~len;
+  check_args ~loc ~pos ~len bstr;
   let min_len = check_min_len ~loc ~len min_len in
   unsafe_output oc ~min_len ~pos ~len bstr
 
 let really_output oc ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"really_output" bstr ~pos ~len;
+  check_args ~loc:"really_output" ~pos ~len bstr;
   ignore (unsafe_output oc ~min_len:len ~pos ~len bstr)
 
-
-(* Output functions *)
-
 external unsafe_really_write :
   file_descr -> pos : int -> len : int -> t -> unit
   = "bigstring_really_write_stub"
 
 let really_write fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"really_write" bstr ~pos ~len;
+  check_args ~loc:"really_write" ~pos ~len bstr;
   unsafe_really_write fd ~pos ~len bstr
 
 #if defined(MSG_NOSIGNAL)
@@ -229,32 +216,25 @@ external unsafe_really_send_no_sigpipe :
 
 let really_send_no_sigpipe fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"really_send_no_sigpipe" bstr ~pos ~len;
+  check_args ~loc:"really_send_no_sigpipe" ~pos ~len bstr;
   unsafe_really_send_no_sigpipe fd ~pos ~len bstr
-#else
-#warning "MSG_NOSIGNAL not defined; really_send_no_sigpipe"
-#warning "not implemented."
-#warning "Try compiling on Linux?"
-#endif
 
-#if defined(MSG_NOSIGNAL)
-(* CRv2 SW: It seems complex to have the underyling C code return the option.
-   Why not leave the C code simple (returning an int) and translate to an
-   option on the ML side?
-   CRv2 mmottl: oh well, small deal.  The current way saves an
-   intermediate function on the ML side and only adds two short lines
-   to the C-code.
- *)
 external unsafe_send_nonblocking_no_sigpipe :
-  file_descr -> pos : int -> len : int -> t -> int option
+  file_descr -> pos : int -> len : int -> t -> int
   = "bigstring_send_nonblocking_no_sigpipe_stub"
 
+let unsafe_send_nonblocking_no_sigpipe fd ~pos ~len buf =
+  let res = unsafe_send_nonblocking_no_sigpipe fd ~pos ~len buf in
+  if res = -1 then None
+  else Some res
+
 let send_nonblocking_no_sigpipe fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"send_nonblocking_no_sigpipe" bstr ~pos ~len;
+  check_args ~loc:"send_nonblocking_no_sigpipe" ~pos ~len bstr;
   unsafe_send_nonblocking_no_sigpipe fd ~pos ~len bstr
-#else
-#warning "MSG_NOSIGNAL not defined; really_send_no_sigpipe"
+
+  #else
+#warning "MSG_NOSIGNAL not defined; really_send_no_sigpipe and friends"
 #warning "not implemented."
 #warning "Try compiling on Linux?"
 #endif
@@ -264,17 +244,17 @@ external unsafe_write :
 
 let write fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"write" bstr ~pos ~len;
+  check_args ~loc:"write" ~pos ~len bstr;
   unsafe_write fd ~pos ~len bstr
 
-external unsafe_write_assume_nonblocking :
+external unsafe_write_assume_fd_is_nonblocking :
   file_descr -> pos : int -> len : int -> t -> int
-  = "bigstring_write_assume_nonblocking_stub"
+  = "bigstring_write_assume_fd_is_nonblocking_stub"
 
-let write_assume_nonblocking fd ?(pos = 0) ?len bstr =
+let write_assume_fd_is_nonblocking fd ?(pos = 0) ?len bstr =
   let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"write_assume_nonblocking" bstr ~pos ~len;
-  unsafe_write_assume_nonblocking fd ~pos ~len bstr
+  check_args ~loc:"write_assume_fd_is_nonblocking" ~pos ~len bstr;
+  unsafe_write_assume_fd_is_nonblocking fd ~pos ~len bstr
 
 external unsafe_writev :
   file_descr -> t Unix_ext.IOVec.t array -> int -> int
@@ -292,13 +272,13 @@ let writev fd ?count iovecs =
   let count = get_iovec_count "writev" iovecs count in
   unsafe_writev fd iovecs count
 
-external unsafe_writev_assume_nonblocking :
+external unsafe_writev_assume_fd_is_nonblocking :
   file_descr -> t Unix_ext.IOVec.t array -> int -> int
-  = "bigstring_writev_assume_nonblocking_stub"
+  = "bigstring_writev_assume_fd_is_nonblocking_stub"
 
-let writev_assume_nonblocking fd ?count iovecs =
+let writev_assume_fd_is_nonblocking fd ?count iovecs =
   let count = get_iovec_count "writev_nonblocking" iovecs count in
-  unsafe_writev_assume_nonblocking fd iovecs count
+  unsafe_writev_assume_fd_is_nonblocking fd iovecs count
 
 
 (* Memory mapping *)
@@ -306,56 +286,18 @@ let writev_assume_nonblocking fd ?count iovecs =
 let map_file ~shared fd n = Array1.map_file fd Bigarray.char c_layout shared n
 
 
-(* File I/O *)
-
-let close_on_exc fd exc =
-  begin try close fd with _ -> () end;
-  raise exc
-
-let load_file ?(pos = 0) ?len fname =
-  if pos < 0 then invalid_arg "Bigstring.load_file: pos < 0";
-  let fd = openfile fname [O_RDONLY] 0o600 in
-  try
-    let len =
-      let file_size = (fstat fd).st_size in
-      match len with
-      | None ->
-          let len = file_size - pos in
-          if len < 0 then invalid_arg "Bigstring.load_file: pos > file size";
-          len
-      | Some len ->
-          if len < 0 then invalid_arg "Bigstring.load_file: len < 0";
-          if pos + len > file_size then
-            invalid_arg "Bigstring.load_file: pos + len > file size";
-          len
-    in
-    ignore (lseek fd pos SEEK_SET);
-    let bstr = create len in
-    really_read fd ~len bstr;
-    bstr
-  with exc -> close_on_exc fd exc
-
-let store_file
-      ?(create = true) ?(exclusive = true) ?(append = true) ?(perm = 0o600)
-      fname ?(pos = 0) ?len bstr =
-  let len = get_opt_len bstr ~pos len in
-  check_args ~loc:"store_file" bstr ~pos ~len;
-  let flags = [O_WRONLY] in
-  let flags = if create then O_CREAT :: flags else flags in
-  let flags = if exclusive then O_EXCL :: flags else flags in
-  let flags = if append then O_APPEND :: flags else flags in
-  let fd = openfile fname flags perm in
-  try really_write fd ~pos ~len bstr
-  with exc -> close_on_exc fd exc
-
-
-(* input and output, linux only *)
 #if defined(MSG_NOSIGNAL)
+(* Input and output, linux only *)
 
 external unsafe_sendmsg_nonblocking_no_sigpipe :
   file_descr -> t Unix_ext.IOVec.t array -> int -> int
   = "bigstring_sendmsg_nonblocking_no_sigpipe_stub"
 
+let unsafe_sendmsg_nonblocking_no_sigpipe fd iovecs count =
+  let res = unsafe_sendmsg_nonblocking_no_sigpipe fd iovecs count in
+  if res = -1 then None
+  else Some res
+
 let sendmsg_nonblocking_no_sigpipe fd ?count iovecs =
   let count = get_iovec_count "sendmsg_nonblocking_no_sigpipe" iovecs count in
   unsafe_sendmsg_nonblocking_no_sigpipe fd iovecs count
@@ -364,3 +306,8 @@ let sendmsg_nonblocking_no_sigpipe fd ?count iovecs =
 #warning "not implemented."
 #warning "Try compiling on Linux?"
 #endif
+
+
+(* Destruction *)
+
+external unsafe_destroy : t -> unit = "bigstring_destroy_stub"
