@@ -4,6 +4,12 @@
 #include <unistd.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <assert.h>
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#else
+#warning "No malloc.h found"
+#endif
 
 #include "ocaml_utils.h"
 #include "unix_utils.h"
@@ -19,6 +25,16 @@ CAMLprim value bigstring_init_stub(value __unused v_unit)
   bigstring_exc_IOError = caml_named_value("Bigstring.IOError");
   bigstring_exc_End_of_file = caml_named_value("Bigstring.End_of_file");
   unix_error_exn = caml_named_value("Unix.Unix_error");
+  /* GLIBC uses a threshold internally as a cutoff between brk and mmap.
+     Sadly, it nowadays employs a heuristic that may change this value
+     dynamically.  The call to mallopt suppresses this behavior, which
+     made it hard to prevent C-heap fragmentation (e.g. in the writer).
+  */
+#ifdef HAVE_MALLOC_H
+  mallopt(M_MMAP_THRESHOLD, 131072);
+#else
+#warning "no malloc.h, not calling mallopt"
+#endif
   if (unix_error_exn == NULL)
     caml_invalid_argument(
       "Exception Unix.Unix_error not initialized, please link unix.cma");
@@ -29,7 +45,7 @@ CAMLprim value bigstring_init_stub(value __unused v_unit)
 /* Blitting */
 
 CAMLprim value bigstring_blit_string_bigstring_stub(
-  value v_src_pos, value v_str, value v_dst_pos, value v_bstr, value v_len)
+  value v_str, value v_src_pos, value v_bstr, value v_dst_pos, value v_len)
 {
   char *str = String_val(v_str) + Long_val(v_src_pos);
   char *bstr = get_bstr(v_bstr, v_dst_pos);
@@ -38,7 +54,7 @@ CAMLprim value bigstring_blit_string_bigstring_stub(
 }
 
 CAMLprim value bigstring_blit_bigstring_string_stub(
-  value v_src_pos, value v_bstr, value v_dst_pos, value v_str, value v_len)
+  value v_bstr, value v_src_pos, value v_str, value v_dst_pos, value v_len)
 {
   char *bstr = get_bstr(v_bstr, v_src_pos);
   char *str = String_val(v_str) + Long_val(v_dst_pos);
@@ -47,19 +63,16 @@ CAMLprim value bigstring_blit_bigstring_string_stub(
 }
 
 CAMLprim value bigstring_blit_stub(
-  value v_src_pos, value v_src, value v_dst_pos, value v_dst, value v_len)
+  value v_src, value v_src_pos, value v_dst, value v_dst_pos, value v_len)
 {
   struct caml_ba_array *ba_src = Caml_ba_array_val(v_src);
   struct caml_ba_array *ba_dst = Caml_ba_array_val(v_dst);
   char *src = (char *) ba_src->data + Long_val(v_src_pos);
   char *dst = (char *) ba_dst->data + Long_val(v_dst_pos);
-  int len = Long_val(v_len);
-  if
-    (
-      (len > THREAD_IO_CUTOFF)
+  size_t len = Long_val(v_len);
+  if ((len > THREAD_IO_CUTOFF)
       || (ba_src->flags & CAML_BA_MAPPED_FILE)
-      || (ba_dst->flags & CAML_BA_MAPPED_FILE)
-    )
+      || (ba_dst->flags & CAML_BA_MAPPED_FILE))
   {
     Begin_roots2(v_src, v_dst);
     caml_enter_blocking_section();
@@ -130,11 +143,11 @@ CAMLprim value bigstring_read_stub(
   value v_min_len, value v_fd, value v_pos, value v_len, value v_bstr)
 {
   CAMLparam1(v_bstr);
-    int min_len = Long_val(v_min_len);
+    size_t min_len = Long_val(v_min_len);
     int fd = Int_val(v_fd);
-    int init_len = Long_val(v_len);
-    int len = init_len;
-    int n_read;
+    size_t init_len = Long_val(v_len);
+    size_t len = init_len;
+    ssize_t n_read;
     char *bstr_start = get_bstr(v_bstr, v_pos);
     char *bstr = bstr_start;
     char *bstr_min = bstr_start + min_len;
@@ -142,10 +155,10 @@ CAMLprim value bigstring_read_stub(
       do {
         n_read = read(fd, bstr, len);
         if (n_read <= 0) {
-          value v_n_good = Val_int(bstr - bstr_start);
+          value v_n_good = Val_long(bstr - bstr_start);
           caml_leave_blocking_section();
           if (n_read == 0) {
-            if (init_len == 0) CAMLreturn(Val_int(0));
+            if (init_len == 0) CAMLreturn(Val_long(0));
             else raise_eof_io_error(v_n_good);
           }
           else raise_unix_io_error(v_n_good, "read", Nothing);
@@ -155,16 +168,16 @@ CAMLprim value bigstring_read_stub(
         }
       } while (bstr < bstr_min);
     caml_leave_blocking_section();
-  CAMLreturn(Val_int(bstr - bstr_start));
+  CAMLreturn(Val_long(bstr - bstr_start));
 }
 
-CAMLprim value bigstring_read_assume_nonblocking_stub(
+CAMLprim value bigstring_read_assume_fd_is_nonblocking_stub(
   value v_fd, value v_pos, value v_len, value v_bstr)
 {
   struct caml_ba_array *ba = Caml_ba_array_val(v_bstr);
   char *bstr = (char *) ba->data + Long_val(v_pos);
-  int len = Long_val(v_len);
-  int n_read;
+  size_t len = Long_val(v_len);
+  ssize_t n_read;
   if ((len > THREAD_IO_CUTOFF) || (ba->flags & CAML_BA_MAPPED_FILE)) {
     Begin_roots1(v_bstr);
     caml_enter_blocking_section();
@@ -173,9 +186,8 @@ CAMLprim value bigstring_read_assume_nonblocking_stub(
     End_roots();
   }
   else n_read = read(Int_val(v_fd), bstr, len);
-  if (n_read == -1)
-    uerror("bigstring_read_assume_nonblocking", Nothing);
-  return Val_int(n_read);
+  if (n_read == -1) uerror("bigstring_read_assume_fd_is_nonblocking", Nothing);
+  return Val_long(n_read);
 }
 
 
@@ -184,28 +196,28 @@ CAMLprim value bigstring_read_assume_nonblocking_stub(
 CAMLprim value bigstring_really_recv_stub(
   value v_sock, value v_pos, value v_len, value v_bstr)
 {
-  int len = Int_val(v_len);
+  size_t len = Long_val(v_len);
   if (len == 0) return Val_unit;
   else {
     CAMLparam1(v_bstr);
       char *bstr = get_bstr(v_bstr, v_pos);
       int sock = Int_val(v_sock);
-      int n_read, n_total = 0;
+      ssize_t n_read;
+      size_t n_total = 0;
       caml_enter_blocking_section();
-loop:
+        while (len > 0) {
           n_read = recv(sock, bstr, len, MSG_WAITALL);
           if (n_read <= 0) {
-            value v_n_total = Val_int(n_total);
+            value v_n_total = Val_long(n_total);
             caml_leave_blocking_section();
             if (n_read == 0) raise_eof_io_error(v_n_total);
-            raise_unix_io_error(v_n_total, "really_recv", Nothing);
-          }
-          len -= n_read;
-          if (len > 0) {
+            else raise_unix_io_error(v_n_total, "really_recv", Nothing);
+          } else {
+            len -= n_read;
             bstr += len;
             n_total += n_read;
-            goto loop;
           }
+        }
       caml_leave_blocking_section();
     CAMLreturn(Val_unit);
   }
@@ -251,14 +263,14 @@ CAMLprim value bigstring_input_stub(
     struct channel *chan = Channel(v_chan);
     char *bstr_start = get_bstr(v_bstr, v_pos);
     char *bstr = bstr_start;
-    int init_bstr_len = Long_val(v_len);
-    int bstr_len = init_bstr_len;
-    int min_len = Long_val(v_min_len);
-    int avail = chan->max - chan->curr;
+    size_t init_bstr_len = Long_val(v_len);
+    size_t bstr_len = init_bstr_len;
+    size_t min_len = Long_val(v_min_len);
+    size_t avail = chan->max - chan->curr;
     Lock(chan);
     if (avail) {
       int got_all = bstr_len <= avail;
-      int to_write = got_all ? bstr_len : avail;
+      size_t to_write = got_all ? bstr_len : avail;
       memcpy(bstr, chan->curr, to_write);
       if (got_all) {
         chan->curr += to_write;
@@ -279,7 +291,7 @@ CAMLprim value bigstring_input_stub(
       struct iovec iovecs[2];
       struct iovec *bstr_iov = &iovecs[0];
       struct iovec *buff_iov = &iovecs[1];
-      int n_read;
+      ssize_t n_read;
       bstr_iov->iov_base = bstr;
       bstr_iov->iov_len = bstr_len;
       buff_iov->iov_base = chan->buff;
@@ -293,22 +305,28 @@ CAMLprim value bigstring_input_stub(
             if (errno == EINTR) continue;
             else {
               value v_n_good =
-                Val_int((char *) bstr_iov->iov_base - bstr_start);
+                Val_long((char *) bstr_iov->iov_base - bstr_start);
               /* Set buffer to empty as required */
               chan->curr = chan->max;
               caml_leave_blocking_section();
+              Unlock(chan);
               raise_unix_io_error(v_n_good, "input", Nothing);
             }
           }
           else {
             /* Zero-read: set buffer to empty as required */
+            assert(n_read == 0);
             chan->curr = chan->max;
-            if (n_read == 0 && init_bstr_len == 0) CAMLreturn(Val_int(0));
-            else {
+            if (init_bstr_len == 0) {
+              caml_leave_blocking_section();
+              Unlock(chan);
+              CAMLreturn(Val_long(0));
+            } else {
               /* EOF handling */
               value v_n_good =
-                Val_int((char *) bstr_iov->iov_base - bstr_start);
+                Val_long((char *) bstr_iov->iov_base - bstr_start);
               caml_leave_blocking_section();
+              Unlock(chan);
               raise_eof_io_error(v_n_good);
             }
           }
@@ -330,10 +348,10 @@ CAMLprim value bigstring_input_stub(
               chan->curr = chan->max;
               caml_leave_blocking_section();
               Unlock(chan);
-              CAMLreturn(Val_int(bstr - bstr_start));
+              CAMLreturn(Val_long(bstr - bstr_start));
             }
           } else {
-            /* Unsufficient data */
+            /* Insufficient data */
             bstr_iov->iov_base = bstr;
             bstr_iov->iov_len -= n_read;
           }
@@ -348,9 +366,9 @@ CAMLprim value bigstring_output_stub(
   CAMLparam2(v_chan, v_bstr);
     struct channel *chan = Channel(v_chan);
     char *bstr = get_bstr(v_bstr, v_pos);
-    int bstr_len = Long_val(v_len);
+    size_t bstr_len = Long_val(v_len);
     Lock(chan);
-    if (bstr_len <= chan->end - chan->curr) {
+    if (bstr_len <= (size_t) (chan->end - chan->curr)) {
       /* Buffer can store all data */
       memcpy(chan->curr, bstr, bstr_len);
       chan->curr += bstr_len;
@@ -359,12 +377,12 @@ CAMLprim value bigstring_output_stub(
     } else {
       /* Buffer cannot store all data */
       int fd = chan->fd;
-      int buff_len = chan->curr - chan->buff;
+      size_t buff_len = chan->curr - chan->buff;
       char *bstr_min = bstr + Long_val(v_min_len);
       struct iovec iovecs[2];
       struct iovec *buff_iov = &iovecs[0];
       struct iovec *bstr_iov = &iovecs[1];
-      int written;
+      ssize_t written;
       buff_iov->iov_base = chan->buff;
       buff_iov->iov_len = buff_len;
       bstr_iov->iov_base = bstr;
@@ -385,30 +403,31 @@ CAMLprim value bigstring_output_stub(
             continue;
           } else {
             /* Write (maybe of even one byte only) failed */
-            value v_n_good = Val_int((char *) bstr_iov->iov_base - bstr);
+            value v_n_good = Val_long((char *) bstr_iov->iov_base - bstr);
             chan->curr = chan->buff + buff_len;
             if (buff_len) memmove(chan->buff, buff_iov->iov_base, buff_len);
             caml_leave_blocking_section();
+            Unlock(chan);
             raise_unix_io_error(v_n_good, "output", Nothing);
           }
         } else {
           /* Write successful */
           chan->offset += written;
-          if (buff_len > written) {
+          if (buff_len > (size_t) written) {
             /* Buffer was partially written only; continue */
             buff_iov->iov_base = (char *) buff_iov->iov_base + written;
             buff_len -= written;
             buff_iov->iov_len = buff_len;
           } else {
             /* Buffer is empty now */
-            int bstr_written = written - buff_len;
+            size_t bstr_written = written - buff_len;
             char *new_bstr = (char *) bstr_iov->iov_base + bstr_written;
             if (new_bstr >= bstr_min) {
               /* Sufficient data was sent */
               chan->curr = chan->buff;
               caml_leave_blocking_section();
               Unlock(chan);
-              CAMLreturn(Val_int(new_bstr - bstr));
+              CAMLreturn(Val_long(new_bstr - bstr));
             } else {
               /* Not yet done */
               bstr_iov->iov_base = new_bstr;
@@ -432,8 +451,8 @@ CAMLprim value bigstring_output_stub(
   { \
     CAMLparam1(v_bstr); \
       int fd = Int_val(v_fd); \
-      int len = Long_val(v_len); \
-      int written; \
+      size_t len = Long_val(v_len); \
+      ssize_t written; \
       char *bstr_start = get_bstr(v_bstr, v_pos); \
       char *bstr = bstr_start; \
       char *bstr_max = bstr + len; \
@@ -441,9 +460,9 @@ CAMLprim value bigstring_output_stub(
         do { \
           CALL_WRITE; \
           if (written == -1) { \
-            value v_n_good = Val_int(bstr - bstr_start); \
+            value v_n_good = Val_long(bstr - bstr_start); \
             caml_leave_blocking_section(); \
-            raise_unix_io_error(v_n_good, XSTR(really_##NAME), Nothing); \
+            raise_unix_io_error(v_n_good, STR(really_##NAME), Nothing); \
           }; \
           len -= written; \
           bstr += written; \
@@ -459,22 +478,22 @@ CAMLprim value bigstring_write_stub(
 {
   CAMLparam1(v_bstr);
   char *bstr = get_bstr(v_bstr, v_pos);
-  int len = Long_val(v_len);
-  int written;
+  size_t len = Long_val(v_len);
+  ssize_t written;
   caml_enter_blocking_section();
     written = write(Int_val(v_fd), bstr, len);
   caml_leave_blocking_section();
   if (written == -1) uerror("write", Nothing);
-  CAMLreturn(Val_int(written));
+  CAMLreturn(Val_long(written));
 }
 
-CAMLprim value bigstring_write_assume_nonblocking_stub(
+CAMLprim value bigstring_write_assume_fd_is_nonblocking_stub(
   value v_fd, value v_pos, value v_len, value v_bstr)
 {
   struct caml_ba_array *ba = Caml_ba_array_val(v_bstr);
   char *bstr = (char *) ba->data + Long_val(v_pos);
-  int len = Long_val(v_len);
-  int written;
+  size_t len = Long_val(v_len);
+  ssize_t written;
   if ((len > THREAD_IO_CUTOFF) || (ba->flags & CAML_BA_MAPPED_FILE)) {
     Begin_roots1(v_bstr);
     caml_enter_blocking_section();
@@ -483,23 +502,30 @@ CAMLprim value bigstring_write_assume_nonblocking_stub(
     End_roots();
   }
   else written = write(Int_val(v_fd), bstr, len);
-  if (written == -1) uerror("write_assume_nonblocking", Nothing);
-  return Val_int(written);
+  if (written == -1) uerror("write_assume_fd_is_nonblocking", Nothing);
+  return Val_long(written);
 }
 
-CAMLprim value bigstring_writev_stub(value v_fd, value v_iovecs, value v_count)
+static inline ssize_t writev_in_blocking_section(
+  value v_fd, value v_iovecs, struct iovec *iovecs, int count)
 {
+  ssize_t ret;
   CAMLparam1(v_iovecs);  /* To protect bigstrings outside of OCaml lock */
-  int count = Long_val(v_count);
-  long total_len = 0;
-  struct iovec *iovecs = copy_iovecs(&total_len, v_iovecs, count);
-  int ret;
   caml_enter_blocking_section();
     ret = writev(Int_val(v_fd), iovecs, count);
     free(iovecs);
   caml_leave_blocking_section();
+  CAMLreturn(ret);
+}
+
+CAMLprim value bigstring_writev_stub(value v_fd, value v_iovecs, value v_count)
+{
+  int count = Int_val(v_count);
+  size_t total_len = 0;
+  struct iovec *iovecs = copy_iovecs(&total_len, v_iovecs, count);
+  ssize_t ret = writev_in_blocking_section(v_fd, v_iovecs, iovecs, count);
   if (ret == -1) uerror("writev", Nothing);
-  CAMLreturn(Val_int(ret));
+  return Val_long(ret);
 }
 
 __pure static inline int contains_mmapped(value v_iovecs, int n)
@@ -512,29 +538,22 @@ __pure static inline int contains_mmapped(value v_iovecs, int n)
   return 0;
 }
 
-/* CR sweeks: Couldn't this share code with bigstring_writev_stub for the case
-   where we decide to release the OCaml lock? */
-CAMLprim value bigstring_writev_assume_nonblocking_stub(
+CAMLprim value bigstring_writev_assume_fd_is_nonblocking_stub(
   value v_fd, value v_iovecs, value v_count)
 {
-  int count = Long_val(v_count);
-  long total_len = 0;
+  int count = Int_val(v_count);
+  size_t total_len = 0;
   struct iovec *iovecs = copy_iovecs(&total_len, v_iovecs, count);
-  int ret;
-  if (total_len > THREAD_IO_CUTOFF || contains_mmapped(v_iovecs, count)) {
-    Begin_roots1(v_iovecs);
-    caml_enter_blocking_section();
-      ret = writev(Int_val(v_fd), iovecs, count);
-      free(iovecs);
-    caml_leave_blocking_section();
-    End_roots();
-  }
+  ssize_t ret;
+  if (total_len > THREAD_IO_CUTOFF || contains_mmapped(v_iovecs, count))
+    /* NOTE: writev_in_blocking_section frees iovecs */
+    ret = writev_in_blocking_section(v_fd, v_iovecs, iovecs, count);
   else {
     ret = writev(Int_val(v_fd), iovecs, count);
     free(iovecs);
   }
-  if (ret == -1) uerror("writev_assume_nonblocking", Nothing);
-  return Val_int(ret);
+  if (ret == -1) uerror("writev_assume_fd_is_nonblocking", Nothing);
+  return Val_long(ret);
 }
 
 #ifdef MSG_NOSIGNAL
@@ -547,28 +566,21 @@ CAMLprim value bigstring_send_nonblocking_no_sigpipe_stub(
   value v_fd, value v_pos, value v_len, value v_bstr)
 {
   char *bstr = get_bstr(v_bstr, v_pos);
-  int ret =
+  ssize_t ret =
     send(Int_val(v_fd), bstr, Long_val(v_len), nonblocking_no_sigpipe_flag);
-  value v_res;
-  if (ret == -1) {
-    if (errno != EAGAIN && errno != EWOULDBLOCK)
-      uerror("send_nonblocking_no_sigpipe", Nothing);
-    return Val_int(0);
-  }
-  v_res = caml_alloc_small(1, 0);
-  Field(v_res, 0) = Val_int(ret);
-  return v_res;
+  if (ret == -1 && errno != EAGAIN && errno != EWOULDBLOCK)
+    uerror("send_nonblocking_no_sigpipe", Nothing);
+  return Val_long(ret);
 }
 
 CAMLprim value bigstring_sendmsg_nonblocking_no_sigpipe_stub(
   value v_fd, value v_iovecs, value v_count)
 {
-  CAMLparam1(v_iovecs);  /* To protect bigstrings outside of OCaml lock */
-  int count = Long_val(v_count);
-  long total_len = 0;
+  int count = Int_val(v_count);
+  size_t total_len = 0;
   struct iovec *iovecs = copy_iovecs(&total_len, v_iovecs, count);
   struct msghdr msghdr = { NULL, 0, NULL, 0, NULL, 0, 0 };
-  int ret;
+  ssize_t ret;
   if (total_len > THREAD_IO_CUTOFF || contains_mmapped(v_iovecs, count)) {
     Begin_roots1(v_iovecs);
     caml_enter_blocking_section();
@@ -586,9 +598,42 @@ CAMLprim value bigstring_sendmsg_nonblocking_no_sigpipe_stub(
   }
   if (ret == -1 && errno != EAGAIN && errno != EWOULDBLOCK)
     uerror("sendmsg_nonblocking_no_sigpipe", Nothing);
-  CAMLreturn(Val_int(ret));
+  return Val_long(ret);
 }
 #else
 #warning "MSG_NOSIGNAL not defined; bigstring_send{,msg}_noblocking_no_sigpipe not implemented"
 #warning "Try compiling on Linux?"
 #endif
+
+/* Destruction */
+
+static void check_bigstring_proxy(struct caml_ba_array *b)
+{
+  if (b->proxy != NULL) caml_failwith("bigstring_destroy: bigstring has proxy");
+}
+
+extern void caml_ba_unmap_file(void *addr, uintnat len);
+
+CAMLprim value bigstring_destroy_stub(value v_bstr)
+{
+  struct caml_ba_array *b = Caml_ba_array_val(v_bstr);
+  int i;
+  switch (b->flags & CAML_BA_MANAGED_MASK) {
+    case CAML_BA_EXTERNAL :
+      caml_failwith(
+        "bigstring_destroy: bigstring is external or already deallocated");
+      break;
+    case CAML_BA_MANAGED :
+      check_bigstring_proxy(b);
+      free(b->data);
+      break;
+    case CAML_BA_MAPPED_FILE :
+      check_bigstring_proxy(b);
+      caml_ba_unmap_file(b->data, caml_ba_byte_size(b));
+      break;
+  }
+  b->data = NULL;
+  b->flags = CAML_BA_EXTERNAL;
+  for (i = 0; i < b->num_dims; ++i) b->dim[i] = 0;
+  return Val_unit;
+}
