@@ -3,6 +3,7 @@ type t = out_channel
 val stdout : t
 val stderr : t
 
+
 val create : ?binary:bool -> ?append:bool -> ?perm:int -> string -> t
 
 val close : t -> unit
@@ -12,6 +13,7 @@ val set_binary_mode : t -> bool -> unit
 
 val flush : t -> unit
 
+
 val output : t -> buf:string -> pos:int -> len:int -> unit
 val output_string : t -> string -> unit
 val output_char : t -> char -> unit
@@ -19,6 +21,9 @@ val output_byte : t -> int -> unit
 val output_binary_int : t -> int -> unit
 val output_value : t -> 'a -> unit
 
-val seek : t -> int -> unit
-val pos : t -> int
-val length : t -> int
+val newline : t -> unit
+val output_lines : t -> string list -> unit
+
+val seek : t -> int64 -> unit
+val pos : t -> int64
+val length : t -> int64
